// File: testPrime.java
// Demonstrates the use of threads to test a number for primality
//	by employing a simplistic divide and conquer strategy
//	Inputs the number to check

// Prepared by Dr. Spiegel
 
public class testPrime {
 
   public static void main(String args[]) {
        if (args.length!=1)	// No input number provided
        {
          System.out.println("invoke like this: java testPrime 1027");
          System.exit(1);
        }
        long possPrime = Long.parseLong(args[0]); // convert arg to int

	// Divide into number of 100's in input value
        int centuries = (int)(possPrime/100) +1;

	// One thread for each 100 values
        testRange [] t = new testRange[centuries];

        for(int i=0;i<centuries;i++) {
	     // Create and start thread for each 100 values
             t[i] = new testRange(possPrime, i*100);
             t[i].start();     
        }
	boolean factorFound=false;
	// We don't test the numbers from the last 100 to possPrime.
	//	Why don't we have to?
        for(int i = 0; i < centuries; i++)
        {  // For each set of 100 values, 
	   try {
	     //	Wait for the thread to die
	     t[i].join(); 
	   }
           catch(InterruptedException ie) {}
	   // Get the value returned
	   long factor=t[i].getFactor();
	   // Report if factor found
	   if (factor>0) {
	     factorFound=true;
             System.out.println(" Thread " + i + " found factor " + factor);
	   }
        }
	if (!factorFound)
	   System.out.println(possPrime + " is prime");
   }
}

class testRange extends Thread {
 
   static long possPrime;
   long from, to;
   long factor = -1;
   // constructor
   //   record the number we are to test, and
   //   the range of factors we are to try.
   testRange(long argpossPrime, int argFrom) 
   {
       possPrime=argpossPrime;
       from=((argFrom==0)?2:argFrom);
       to=argFrom+99;
   }
   long getFactor(){return factor;}
   public void run() {
      for (long i=from; i<=to && i<possPrime; i++) 
      {
         if (possPrime%i == 0)
         {  // i divides possPrime exactly
                System.out.println("factor "+i+" found by thread "+getName());
                factor = i;
                this.stop();
         }
         yield();
      }   
   }
 
}
 

